/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.hayabusa.taglib;

import static org.opengion.fukurou.util.StringUtil.nval;

import java.io.File;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.XHTMLTag;
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.resource.GUIInfo;
import org.opengion.hayabusa.resource.URLXfer;


/**
 * HTML のaタグと同等のリンクを作成するHTML拡張タグです。
 *
 * @og.formSample
 * ●形式：&lt;og:link href="…" &gt; &lt;jsp:text&gt; ... &lt;/jsp:text&gt; &lt;/og:link &gt;
 * ●body：あり
 *
 * ●使用例
 *    ・普通のリンク
 *        &lt;og:link gamenId="32ExcelIn" href="direct.jsp" keys="location,filename,modifyType" value="{&#064;GUI.ADDRESS},{&#064;GUI.ADDRESS}.xls,A"  target="RESULT" &gt;
 *            &lt;img src="{&#064;SYS.JSP}/image/insert.gif" alt="Excel Insert" /&gt;&lt;og:message msg="MSG0057" /&gt;
 *        &lt;/og:link&gt;
 *
 *    ・検索結果テーブル内でのリンク
 *        &lt;og:viewLink command="{&#064;command}"&gt;
 *            &lt;og:link column="KMZ" gamenId="RK0251" href="index.jsp" target="CONTENTS"
 *                keys="KMZ,ORDER_BY,command" vals="[KMZ],'LKIS',NEW" /&gt;
 *        &lt;/og:viewLink&gt;
 *
 *    ・ダイレクトジャンプ(指定の画面へ直接飛ばす)URL
 *      gamenId を指定しなければこの画面へのURLを作成します。
 *      このURLは、その画面の index.jsp に直接ジャンプするためのURLです。
 *      この場合、useRequestCache="true" が初期値なので、必要なければ、false に設定願います。
 *        &lt;og:link useDirectJump="true" delKeys="pageSize"
 *                     keys="location,filename,modifyType" value="{&#064;GUI.ADDRESS},{&#064;GUI.ADDRESS}.xls,A" /&gt;
 * @og.group 画面表示
 *
 * @version  4.0
 * @author	 Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class LinkTag extends HTMLTagSupport {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0 (2005/08/31)" ;

	private static final long serialVersionUID = 4000 ;	// 4.0.0 (2005/01/31)
	
	private String		column		= null;
	private String		onLink		= "true";	// true または 1
	private boolean		fileCheck	= false;
	private boolean		filePath	= false;
	private String		baseURL		= HybsSystem.sys( "FILE_URL" );
	private String		fileURL		= baseURL;
	private String		gamenId		= null;
	private boolean		useGamenId	= true;
	// 3.5.2.0 (2003/10/20)
	private String		markList	= "true|TRUE|1";	// true または 1
	private boolean		xlink		= false;			// 3.5.5.0 (2004/03/12)
	private boolean		useDirectJump = false;			// 3.7.1.0 (2005/04/26)
	private boolean		useRequestCache = true;			// 3.7.1.0 (2005/04/26)
	private int			validDuration = -1;
	private boolean		hrefOnly	= false;			// 4.0.0 (2005/11/30)

	private String[]	keys		= null;		// 3.5.6.2 (2004/07/05)
	private String[]	vals		= null;		// 3.5.6.2 (2004/07/05)
	private String[]	delKeys		= null;		// 3.7.1.0 (2005/04/26)
	private boolean		useURLCheck	= false;	 	// 4.3.7.0 (2009/06/01)
	private String		urlCheckUser= null;			// 4.3.7.0 (2009/06/01)
	private long		urlCheckTime= 1000*60*60*24;// 4.3.7.0 (2009/06/01)
	
	/**
	 * Taglibの開始タグが見つかったときに処理する doStartTag() を オーバーライドします。
	 *
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。ボディが存在する場合のみボディが呼ばれる対応。
	 *
	 * @return	int
	 */
	@Override
	public int doStartTag() {
		String msg = getMsglbl();
		if( msg == null && column != null ) { msg = "[" + column + "]"; }
		set( "body",msg );
		return( EVAL_BODY_BUFFERED );	// Body を評価する。（ extends BodyTagSupport 時）
	}

	/**
	 * Taglibのタグ本体を処理する doAfterBody() を オーバーライドします。
	 *
	 * @og.rev 3.1.1.0 (2003/03/28) ボディの内容を取得する処理を、CommonTagSupport で行う。
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。ボディが存在する場合のみボディが呼ばれる対応。
	 *
	 * @return	int
	 */
	@Override
	public int doAfterBody() {
		String label = getBodyString();

		if( label != null && label.length() > 0 ) {
			set( "body",label );
		}

		return(SKIP_BODY);
	}

	/**
	 * Taglibの終了タグが見つかったときに処理する doEndTag() を オーバーライドします。
	 *
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。release2() を doEndTag()で呼ぶ。
	 * @og.rev 3.1.2.0 (2003/04/07) リンク指定時に、画面IDを渡して、実アドレスを求める事ができる様にする。
	 * @og.rev 3.1.7.0 (2003/05/02) 戻るボタン用のBACK_GAMENID を、引数に追加します。
	 * @og.rev 3.1.8.0 (2003/05/16) GAMENID属性のＵＲＬへの自動セットを行うかどうかを決める属性( useGamenId )を追加。
	 * @og.rev 3.1.8.0 (2003/05/16) 画面ＩＤが指定されたときのリンク情報作成方法を通常のリンク時にも適用する。
	 * @og.rev 3.1.8.0 (2003/05/16) 画面のロールに応じて、リンクを張る/張らないを自動判断する。
	 * @og.rev 3.5.2.0 (2003/10/20) markList属性を追加
	 * @og.rev 3.5.5.0 (2004/03/12) xlink属性をAttributeオブジェクトに追加
	 * @og.rev 3.5.6.2 (2004/07/05) makeTag() の処理範囲変更、viewlink へのパラメータ変更。
	 * @og.rev 3.7.1.0 (2005/04/26) ダイレクトジャンプリンク機能を追加
	 * @og.rev 4.0.0 (2005/11/30) roles 属性の追加
	 * @og.rev 4.0.0 (2005/08/31) 同一カラムの複数登録を許可します。
	 * @og.rev 4.3.0.0 (2008/07/04) ロールモードマルチ対応
	 * @og.rev 4.3.7.1 (2009/06/08) URLチェック機能対応
	 * @og.rev 4.3.7.4 (2009/07/01) 循環参照を解消
	 * @og.rev 4.3.8.0 (2009/08/01) ユーザーIDの外部設定に対応
	 *
	 * @return	int
	 */
	@Override
	public int doEndTag() {
		debugPrint();		// 4.0.0 (2005/02/28)
		// 3.1.8.0 (2003/05/16) GAMENID属性のＵＲＬへの自動セットを行うかどうかを決める属性( useGamenId )を追加。

		// 4.0.0 (2005/11/30) 指定のロールが、アクセス許可の場合は、出力します。
		String roles = get( "roles" );
//		if( roles != null && ! getUser().isAccess( roles ) ) {
//			return(EVAL_PAGE);
//		}
		// 4.3.0.0 (2008/07/04) ロールモードマルチ対応
		if( ! getUser().isAccess( roles ) ) {
			return(EVAL_PAGE);
		}

		//  4.3.7.1 (2009/06/08)
		if( useURLCheck ) {
			// 4.3.8.0 (2009/08/01)
//			urlCheckUser = getUser().getUserID();
			urlCheckUser = nval( urlCheckUser, getUser().getUserID() );
		}

		ViewLinkTag viewlink = (ViewLinkTag)findAncestorWithClass( this,ViewLinkTag.class );
		if( viewlink != null ) {
			if( isGamenCheck() ) {	set( "onLink", onLink );  }
			else {					set( "onLink", "false" ); }
			 // 4.3.7.1 (2009/06/08)
			if( useURLCheck ) {
				set( "useURLCheck", "true" );
				set( "urlCheckUser", urlCheckUser );
				set( "urlCheckTime", String.valueOf( System.currentTimeMillis() + urlCheckTime ) );
			}
			set( "markList",markList );
			set( "linkFormat",makeTag() );		// 3.5.6.2 (2004/07/05)
			// 4.0.0 (2005/08/31) 同一カラムの複数登録を許可します。
			set( "column",column );
			viewlink.addAttribute( getAttributes() );
		}
		// 3.7.1.0 (2005/04/26) ダイレクトジャンプリンク機能を追加
		// 画面の利用権限有無などは調査不要
		else if( useDirectJump ) {
			jspPrint( makeDirectLink() );
		}
		else {
			if( isFileCheck() && isGamenCheck() && markList.indexOf( onLink ) >= 0 ) {
				// 3.5.6.2 (2004/07/05) makeTag() より移動
				if( filePath ) {
					String furl  = get( "href" );

					fileURL = StringUtil.urlAppend( baseURL,furl );
					String filename =  HybsSystem.url2dir( fileURL );

					set( "href",filename );
				}

				// 4.3.7.1 (2009/06/08)
				String tag = makeTag();
				if( useURLCheck ) {
					if( hrefOnly ) {
						// 4.3.7.4 (2009/07/01) 循環参照を解消
						tag = XHTMLTag.addURLCheckKey( tag, HybsSystem.URL_CHECK_KEY, getUser().getUserID(), System.currentTimeMillis() + urlCheckTime );
					}
					else {
						// 4.3.7.4 (2009/07/01) 循環参照を解消
						tag = XHTMLTag.embedURLCheckKey( tag, HybsSystem.URL_CHECK_KEY, getUser().getUserID(), System.currentTimeMillis() + urlCheckTime );
					}
				}
				jspPrint( tag );
//				jspPrint( makeTag() );
			}
		}

		return(EVAL_PAGE);
	}

	/**
	 * タグリブオブジェクトをリリースします。
	 * キャッシュされて再利用されるので、フィールドの初期設定を行います。
	 *
	 * @og.rev 2.0.0.4 (2002/09/27) カスタムタグの release() メソッドを、追加
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。release2() を doEndTag()で呼ぶ。
	 * @og.rev 3.1.2.0 (2003/04/07) リンク指定時に、画面IDを渡して、実アドレスを求める事ができる様にする。
	 * @og.rev 3.1.8.0 (2003/05/16) GAMENID属性のＵＲＬへの自動セットを行うかどうかを決める属性( useGamenId )を追加。
	 * @og.rev 3.5.2.0 (2003/10/20) markList属性を追加
	 * @og.rev 3.5.5.0 (2004/03/12) xlink属性を追加
	 * @og.rev 3.5.6.2 (2004/07/05) keys , vals属性を追加
	 * @og.rev 3.7.1.0 (2005/04/26) useDirectJump , useRequestCache , delKeys 属性を追加
	 * @og.rev 4.0.0 (2005/09/30) validDuration 属性を追加
	 * @og.rev 4.3.7.0 (2009/06/01) useURLCheck属性を追加
	 *
	 */
	@Override
	protected void release2() {
		super.release2();
		column		= null;
		onLink		= "true";	// true または 1
		markList	= "true|TRUE|1";	// 3.5.2.0 (2003/10/20)
		fileCheck	= false;
		filePath	= false;
		baseURL		= HybsSystem.sys( "FILE_URL" );
		fileURL		= baseURL;
		gamenId		= null;
		useGamenId	= true;
		xlink		= false;	// 3.5.5.0 (2004/03/12)
		keys		= null;		// 3.5.6.2 (2004/07/05)
		vals		= null;		// 3.5.6.2 (2004/07/05)
		useDirectJump = false;	// 3.7.1.0 (2005/04/26)
		useRequestCache = true;	// 3.7.1.0 (2005/04/26)
		delKeys		= null;		// 3.7.1.0 (2005/04/26)
		validDuration = -1;
		hrefOnly	= false;	// 4.0.0 (2005/11/30)
		useURLCheck	= false;	// 4.3.7.0 (2009/06/01)
		urlCheckUser= null;		// 4.3.7.0 (2009/06/01)
		urlCheckTime= 1000*60*60*24;	// 4.3.7.0 (2009/06/01)
	}

	/**
	 * リンクを作成します。
	 *
	 * @og.rev 3.5.5.0 (2004/03/12) xlink 属性に応じたリンク文字を作成します。
	 * @og.rev 3.5.6.2 (2004/07/05) XHTMLTag の変換値のみを返す様に変更。
	 * @og.rev 3.7.0.3 (2005/03/01) リンク作成時に行番号を引数に追加する。
	 * @og.rev 4.0.0 (2005/11/30) href 属性のみを返します。
	 * @og.rev 4.0.1.0 (2007/12/17) BACK_ADDRESSの追加
	 * @og.rev 4.3.3.5 (2008/11/08) サーバーアドレスを取得する際に、リクエストから取得するように変更します。
	 * @og.rev 4.3.7.0 (2009/06/01) URLチェック機能対応
	 * @og.rev 4.3.7.1 (2009/06/08) ↑の実装を#doEndTagに移動
	 *
	 * @return	リンクタグ文字列
	 */
	protected String makeTag() {

		makeGamenLink();		// 3.5.6.2 (2004/07/05)
		if( get( "body" ) == null ) { set( "body" , get( "href" ) ); }

		// xlink と通常の link では、URL接続文字が異なる。
		String sep = ( xlink ) ? "&amp;" : "&" ;

		String urlEnc = XHTMLTag.urlEncode( keys,vals,sep );
		// 3.5.6.2 (2004/07/05) addBackGamenId() メソッドの内容を移動
		if( useGamenId ) {
			String backGamenId = getGUIInfoAttri( "KEY" );	// 4.0.0 (2005/01/31)
			urlEnc += sep + "BACK_ROW={I}" + sep + "BACK_GAMENID=" + backGamenId ;	// 3.7.0.3 (2005/03/01)
			if( gamenId != null && gamenId.length() > 0 ) {
				urlEnc += sep + "GAMENID=" + gamenId ;
			}
			// 4.0.1.0 (2007/12/17) gamenIdがある場合はbackAddressを付加する。（BackGamenTagではこれで戻る）
			String guiAddress = getGUIInfoAttri( "ADDRESS" );

			// 4.3.3.5 (2008/11/08) サーバーアドレスを取得する際に、リクエストから取得するように変更します。
//			String backAddress = sys("CONTEXT_URL") + "jsp/"+guiAddress+"/index.jsp";
			String backAddress = getContextURL() + guiAddress + "/index.jsp";
			urlEnc += sep + "BACK_ADDRESS=" + backAddress;
		}

		// 4.0.0 (2005/11/30) href 属性のみを返します。
		if( hrefOnly ) {
			return XHTMLTag.addUrlEncode( get( "href" ),urlEnc,sep );
		}

		if( xlink ) {
			return XHTMLTag.xlink( getAttributes(),urlEnc ) ;
		}
		else {
			return XHTMLTag.link( getAttributes(),urlEnc ) ;
		}
	}

	/**
	 * ダイレクトジャンプリンクを作成します。
	 *
	 * gamenId を指定しなければ、この画面が指定されたと考えます。
	 * target を指定しなければ、"_blank" を指定します。
	 * msg がなければ(または、BODY部が空の場合)、画面名にこのリンクを作成します。
	 *
	 * @og.rev 3.7.1.0 (2005/04/26) 新規追加
	 * @og.rev 4.0.0 (2005/09/30) リンク文字列をハッシュ化します。
	 * @og.rev 4.3.3.5 (2008/11/08) サーバーアドレスを取得する際に、リクエストから取得するように変更します。
	 *
	 * @return	ダイレクトジャンプリンク文字列
	 */
	private String makeDirectLink() {

	 	// gamenId を指定しなければ、この画面が指定されたと考えます。
		if( gamenId == null ) {
			gamenId = getGUIInfoAttri( "KEY" );
		}

	 	// キャッシュ情報と keys,vals情報を設定します。
		if( useRequestCache ) { makeRequestCache2Vals( gamenId ); }
		String urlEnc = XHTMLTag.urlEncode( keys,vals );
		urlEnc = urlEnc + "&GAMENID=" + gamenId ;

		String href = get( "href" );
		if( href == null || href.length() == 0 ) {
			// 4.3.3.5 (2008/11/08) サーバーアドレスを取得する際に、リクエストから取得するように変更します。
//			href = sys( "CONTEXT_URL" ) + "jsp/index.jsp" ;
			href = getContextURL() + "index.jsp" ;
		}

		String redirectURL = href + "?" + urlEnc ;
		String name = getGUIInfoAttri( "LNAME" );

		String validDate = null;	// 設定されない場合は、無制限
		if( validDuration != 0 ) {
			if( validDuration < 0 ) {	// マイナスの場合は、初期値が使われます。
				validDuration = sysInt( "LINK_VALID_DURATION_DAYS" );
			}
			                                        //  ミリ秒    秒    時間
			long nowTime = System.currentTimeMillis() + 1000L * 3600L * 24L * validDuration ;
			validDate = HybsSystem.getDate( nowTime,"yyyyMMdd" );
		}
		String userid   = getUser().getUserID();

		URLXfer xfer = new URLXfer();
		// redirectURL,name,validDate,userid
		return xfer.getRandomURL( redirectURL,name,validDate,userid );
	}

	/**
	 * リクエストキャッシュ情報を加味した keys,vals を再作成します。
	 * このメソッドは、ダイレクトジャンプ時のみ有効です。
	 *
	 * リクエスト情報のMapを取り出します。そのとき、keys,vals を優先して
	 * 設定します。
	 *
	 * @og.rev 3.7.1.0 (2005/04/26) 新規追加
	 *
	 * @param	gamenId String 画面ID(リクエストキャッシュの取り出しキー)
	 */
	@SuppressWarnings(value={"unchecked"})
	private void makeRequestCache2Vals( final String gamenId ) {
		Map<String,String[]> cache = new HashMap<String,String[]>();

		String cacheKey = HybsSystem.REQ_CACHE_KEY + gamenId ;
		Map<String,String[]> map = (Map<String,String[]>)getSessionAttribute( cacheKey );
		if( map != null ) { cache.putAll( map ); }

		if( keys != null ) {
			for( int i=0; i<keys.length; i++ ) {
				// keys,vals をキャッシュに上書き( キャッシュは、文字列配列)
				String[] temp = new String[] { vals[i] } ;
				cache.put( keys[i],temp );
			}
		}

		// ダイレクトジャンプするための画面IDは、別途設定するため、削除しておきます。
		cache.remove( "GAMENID" );

		// 使用しないキーを、削除しておきます。
		if( delKeys != null ) {
			for( int i=0; i<delKeys.length; i++ ) {
				cache.remove( delKeys[i] );
			}
		}

		// マップキーを取得。キャッシュ値は、String[] なので、複数キー登録する。
		String[] cKeys = cache.keySet().toArray( new String[cache.size()] );

		ArrayList<String> v_keys = new ArrayList<String>();
		ArrayList<String> v_vals = new ArrayList<String>();
		for( int i=0; i<cKeys.length; i++ ) {
			String   kk = cKeys[i];
			String[] vv = cache.get( kk );
			if( vv != null ) {
				for( int j=0; j<vv.length; j++ ) {
					v_keys.add( kk );
					v_vals.add( vv[j] );
				}
			}
		}

		keys = v_keys.toArray( new String[v_keys.size()] );
		vals = v_vals.toArray( new String[v_vals.size()] );
	}

	/**
	 * ファイルの存在チェックを行います。
	 *
	 * 実ファイルは、URL(httpの引数）より、変換して求めます。
	 *
	 * @og.rev 3.1.8.0 (2003/05/16) 画面のロールに応じて、リンクを張る/張らないを自動判断する。
	 *
	 * @return	ファイルが存在する(true)/存在しない(false)
	 */
	private boolean isFileCheck() {
		boolean rtn = true;

		if( fileCheck ) {
			File file = null;

			String url  = get( "href" );
			if( url != null ) {
				String fin = HybsSystem.url2dir( url );
				file = new File( fin );
			}
			rtn = ( file != null && file.exists() );
		}

		return rtn ;
	}

	/**
	 * リンク先の画面のロールに応じて、リンクを張る/張らないを自動判断を行います。
	 *
	 * リンク先の画面ＩＤは、gamenId 属性で指定してください。それ以外のリンクは、
	 * 自動判断できません。
	 * link タグ単独の場合に、画面にアクセスできない場合は、メッセージも表示されません。
	 * viewLink タグを利用される場合は、リンクのON/OFF で制御します。
	 *
	 * @og.rev 3.1.8.0 (2003/05/16) 画面のロールに応じて、リンクを張る/張らないを自動判断する。
	 * @og.rev 4.0.0 (2005/01/31) GUIオブジェクトがマップに無ければ、アクセスできない。
	 *
	 * @return	画面アクセス許可する(true)/許可しない(false)
	 */
	private boolean isGamenCheck() {
		boolean rtn = true;

		if( gamenId != null ) {
			GUIInfo guiInfo = getGUIInfo( gamenId );		// 4.0.0 (2005/01/31)

			// 4.0.0 (2005/01/31) ユーザー毎にアクセスできるGUIが設定されている為
			// 存在しなければ、アクセス不可ということ。
			if( guiInfo == null ) { rtn = false; }
		}

		return rtn ;
	}

	/**
	 * 画面ＩＤとhref から、指定のＵＲＬを作成します。
	 *
	 * @og.rev 3.1.2.0 (2003/04/07) リンク指定時に、画面IDを渡して、実アドレスを求める事ができる様にする。
	 * @og.rev 3.1.8.0 (2003/05/16) リンク作成時に、kyes/values が null の時の処理の修正。
	 * @og.rev 3.1.8.0 (2003/05/16) GAMENID属性のＵＲＬへの自動セットを行うかどうかを決める属性( useGamenId )を追加。
	 * @og.rev 3.1.8.0 (2003/05/16) 画面ＩＤが指定されたときのリンク情報作成方法を通常のリンク時にも適用する。
	 * @og.rev 3.5.5.0 (2004/03/12) URLを求めるのに、GUIInfo#getRealAddress() を使用する。
	 * @og.rev 4.0.0 (2005/01/31) GUIInfoの実アドレスのパラメータを考慮する。
	 *
	 */
	private void makeGamenLink() {
		if( gamenId == null || gamenId.length() == 0 ) { return ; }	// 4.0.0 (2005/11/30)

		GUIInfo guiInfo = getGUIInfo( gamenId );		// 4.0.0 (2005/01/31)
		if( guiInfo == null ) { return ; }	// 見つからない場合は、アクセス不可

		String address = guiInfo.getRealAddress( get( "href" ) );
		String url = getRequestParameter( address );
		set( "href",url );

		String label = guiInfo.getLabel();
		if( get( "title" ) == null ) {
			set( "title","To:" + label );
		}
		if( get( "body" ) == null ) {
			set( "body",label );
		}
	}

	/**
	 * リクエストからサーバーアドレス（例：http://hn5120:8824/ver4/jsp/）を取得します。
	 * 
	 * @og.rev 4.3.3.5 (2008/11/08) 新規追加。
	 * 
 	 * @return	サーバーアドレス文字列
	 */
	private String getContextURL() {
		StringBuilder link = new StringBuilder( HybsSystem.BUFFER_MIDDLE );
		HttpServletRequest request = (HttpServletRequest) getRequest();
		link.append( "http://" );
		link.append( request.getServerName() ).append( ":" );
		link.append( request.getServerPort() );
		link.append( request.getContextPath() );
		link.append( "/jsp/" );
		return link.toString();
	}
	
	/**
	 * 【HTML】リンク先のMIMEタイプを指定します。
	 *
	 * @og.tag リンク先のMIMEタイプを指定します。
	 *
	 * @param	type String
	 */
	public void setType( final String type ) {
		set( "type",getRequestParameter( type ) );
	}

	/**
	 * 【HTML】この要素をリンクの対象とするための名前を指定します。
	 *
	 * @og.tag この要素をリンクの対象とするための名前を指定します。
	 *
	 * @param	name String
	 */
	public void setName( final String name ) {
		set( "name",getRequestParameter( name ) );
	}

	/**
	 * 【TAG】リンクをつけたいカラムを指定します(ViewLink タグと併用)。
	 *
	 * @og.tag
	 * このカラム名のTableModelに対して、 ViewLink タグが存在する場合は,
	 * リンクを張ります。
	 *
	 * @param	clm String
	 */
	public void setColumn( final String clm ) {
		this.column = getRequestParameter( clm );
	}

	/**
	 * 【HTML】リンク先のURLを指定します。
	 *
	 * @og.tag リンク先のURLを指定します。
	 *
	 * @param	href String
	 */
	public void setHref( final String href ) {
		set( "href",getRequestParameter( href ) );
	}

	/**
	 * 【HTML】リンク先の基本となる言語コードを指定します。
	 *
	 * @og.tag リンク先の基本となる言語コードを指定します。
	 *
	 * @param	hreflang String
	 */
	public void setHreflang( final String hreflang ) {
		set( "hreflang",getRequestParameter( hreflang ) );
	}

	/**
	 * 【HTML】この文書から見た、href属性で指定されるリンク先との関係を指定します。
	 *
	 * @og.tag この文書から見た、href属性で指定されるリンク先との関係を指定します。
	 *
	 * @param	rel String
	 */
	public void setRel( final String rel ) {
		set( "rel",getRequestParameter( rel ) );
	}

	/**
	 * 【HTML】href属性で指定されるリンク先から見た、この文書との関係をしてします。
	 *
	 * @og.tag href属性で指定されるリンク先から見た、この文書との関係をしてします。
	 *
	 * @param	rev String
	 */
	public void setRev( final String rev ) {
		set( "rev",getRequestParameter( rev ) );
	}

	/**
	 * 【HTML】リンク先の文字コードセットを指定します。
	 *
	 * @og.tag リンク先の文字コードセットを指定します。
	 *
	 * @param	charset String
	 */
	public void setCharset( final String charset ) {
		set( "charset",getRequestParameter( charset ) );
	}

	/**
	 * 【HTML】リンク先の表示ターゲットを指定します。
	 *
	 * @og.tag リンク先の文書を表示させるフレーム、またはウィンドウの名前を指定します。
	 *
	 * @param	target String
	 */
	public void setTarget( final String target ) {
		set( "target",getRequestParameter( target ) );
	}

	/**
	 * 【HTML】リンクボタンの形状(イメージマップ rect/circle/poly/default)を指定します。
	 *
	 * @og.tag リンクボタンの形状(イメージマップ rect/circle/poly/default)を指定します。
	 *
	 * @param	shape String
	 */
	public void setShape( final String shape ) {
		set( "shape",getRequestParameter( shape ) );
	}

	/**
	 * 【HTML】リンクボタンの座標を指定します。
	 *
	 * @og.tag
	 * coords セット
	 *
	 * @param	coords String
	 */
	public void setCoords( final String coords ) {
		set( "coords",getRequestParameter( coords ) );
	}

	/**
	 * 【TAG】リンク先に渡すキーをCSV形式で複数指定します。
	 *
	 * @og.tag リンク先に渡すキーを指定します。
	 * 分解方法は、CSV変数を先に分解してから、getRequestParameter で値を取得します。
	 * こうしないとデータ自身にカンマを持っている場合に分解をミスる為です。
	 *
	 * @og.rev 3.5.6.2 (2004/07/05) CommonTagSupport#getCSVParameter を使用
	 *
	 * @param	key String
	 */
	public void setKeys( final String key ) {
		keys = getCSVParameter( key );
	}

//	/**
//	 * 【廃止】リンク先に渡す値をCSV形式で複数指定します。
//	 *
//	 * @og.tag リンク先に渡す値を指定します。
//	 *
//	 * @og.rev 3.5.6.2 (2004/07/05) 廃止します。
//	 * @og.rev 4.0.0 (2007/06/18) 廃止
//	 *
//	 * @param	value String
//	 * @deprecated  vals 属性を使用してください。
//	 */
//	@Deprecated public void setValue( final String value ) {
//		setVals( value );
//	}

	/**
	 * 【TAG】リンク先に渡す値をCSV形式で複数指定します。
	 *
	 * @og.tag リンク先に渡す値を指定します。
	 * 分解方法は、CSV変数を先に分解してから、getRequestParameter で値を取得します。
	 * こうしないとデータ自身にカンマを持っている場合に分解をミスる為です。
	 *
	 * @og.rev 3.5.6.2 (2004/07/05) CommonTagSupport#getCSVParameter を使用
	 *
	 * @param	val String
	 */
	public void setVals( final String val ) {
		vals = getCSVParameter( val );
	}

	/**
	 * 【TAG】リンクを張る(true or 1)/張らない(false or 0)の設定を指定します(初期値:true)。
	 *
	 * @og.tag
	 * リンクを張る場合は、"true"(または "1") リンクを張らない
	 * 場合は, "true以外"(または "0")をセットします。
	 * 初期値は、 "true"(リンクを張る)です。
	 * さらに、[カラム名] で、動的にカラムの値で、リンクを張る、張らないを
	 * 選ぶ事が可能になります。値は、"true"(または "1") で、マークします。
	 * 追記 3.5.2.0 (2003/10/20)：markList属性に、処理対象文字列郡を指定できます。
	 * これにより、リンクを処理するかどうかの判断が、true,1 以外にも使用できるようになりました。
	 *
	 * @og.rev 3.5.2.0 (2003/10/20) markList属性を追加
	 *
	 * @param	flag リンクの設定 "true"(リンクを張る)/その他(リンクを張らない)
	 */
	public void setOnLink( final String flag ) {
		onLink = nval( getRequestParameter( flag ),onLink );
	}

	/**
	 * 【TAG】ファイルの存在チェックを行うかどうか(true/false)を指定します(初期値:false)。
	 *
	 * @og.tag
	 * "true"の場合は、ファイルの存在チェックを行い、ファイルがあれば
	 * リンクを張ります。ファイルが存在しない場合は, 何も表示されません。
	 * 初期値は、 "false"(ファイルチェックを行わない)です。
	 * 注意：現状、viewLink タグからの使用は出来ませんので、ご了承願います。
	 *
	 * @param	flag リンクの設定 "true"(リンクを張る)/その他(リンクを張らない)
	 */
	public void setFileCheck( final String flag ) {
		fileCheck = nval( getRequestParameter( flag ),fileCheck );
	}

	/**
	 * 【TAG】アドレスにファイルのディレクトリ直接のアドレスを指定するかどうか(true/false)設定します(初期値:false)。
	 *
	 * @og.tag
	 * "true"の場合は、ファイルの物理ディレクトリをリンクします。
	 * 初期値は、 "false"(ファイルの直接アドレスを行わない)です。
	 *
	 * @param	flag ファイルのディレクトリ直接のアドレスの指定
	 *           "true"(直接アドレス)/その他(通常のURL)
	 */
	public void setFilePath( final String flag ) {
		filePath = nval( getRequestParameter( flag ),filePath );
	}

	/**
	 * 【TAG】gamenId 属性を登録します。
	 *
	 * @og.tag
	 * gamenId 属性は、画面IDをキーに、実アドレスを求めるのに使用します。
	 * 画面IDが指定された場合は、実アドレスに変換する。指定されない場合は、
	 * href 属性をそのままリンク情報を作成します。
	 * 画面IDを指定する場合の href は、JSPファイル名（例えば、index.jsp 等）です。
	 *
	 * @og.rev 3.1.2.0 (2003/04/07) リンク指定時に、画面IDを渡して、実アドレスを求める事ができる様にする。
	 *
	 * @param   id gamenId 属性
	 */
	public void setGamenId( final String id ) {
		gamenId = nval( getRequestParameter( id ),gamenId );
	}

	/**
	 * 【TAG】GAMENID,BACK_GAMENID 属性を付加するかどうか(true/false)を設定します(初期値:true)。
	 *
	 * @og.tag
	 * "true"の場合は、GAMENID,BACK_GAMENID 属性を href に追加します。(?KEY1=VAL1&amp;KEY2=VAL2･･･)
	 * "false"の場合は、付加しません。
	 * JavaScript や、通常のファイル（ .html や .pdf 等）に、GAMENID,BACK_GAMENID 属性を付けないように
	 * する事が出来ます。
	 * 初期値は、 "true"(GAMENID,BACK_GAMENID 属性を href に追加する)です。
	 *
	 * @og.rev 3.1.8.0 (2003/05/16) GAMENID属性のＵＲＬへの自動セットを行うかどうかを決める属性( useGamenId )を追加。
	 *
	 * @param	flag BACK_GAMENID 属性を付加するかどうか
	 *           "true"(追加する)/その他(追加しない)
	 */
	public void setUseGamenId( final String flag ) {
		useGamenId = nval( getRequestParameter( flag ),useGamenId );
	}

	/**
	 * 【TAG】処理するマークを、"|"区切りの文字列で指定します。
	 *
	 * @og.tag
	 * markListで与えられた文字列に、onLink文字列（大文字/小文字の区別あり）が
	 * 含まれていれば、処理します。
	 * 例えば、"A","B","C" という文字列が、onLink で指定された
	 * 場合に処理するようにしたい場合は、"A|B|C" をセットします。
	 * markList には、[カラム名]指定が可能です。
	 * 初期値は、 "true|TRUE|1"です。
	 *
	 * @og.rev 3.5.2.0 (2003/10/20) 新規追加
	 *
	 * @param	list 処理するマーク(indexOf による含む/含まない判定)
	 */
	public void setMarkList( final String list ) {
		markList = nval( getRequestParameter( list ),markList );
	}

	/**
	 * 【TAG】href ではなく、xlink:href を使用するかどうかを指定します（初期値：false)。
	 *
	 * @og.tag
	 * xlink:href 属性は、XMLでのリンクを指定します。その場合、href 属性で指定した
	 * 内容は、xlink:href に置き換えられます。
	 * また、URL文字列の接続文字列（&）を、(&amp;)で接続します。これは、xlink として、
	 * XMLファイルに埋め込む場合に、置き換えが必要な為です。
	 * 初期値は、 "false"（通常の href） です。
	 *
	 * @og.rev 3.5.5.0 (2004/03/12) 新規追加
	 *
	 * @param	lnk XMLリンクを使用するかどうか（使用する：true/使用しない：false)
	 */
	public void setXlink( final String lnk ) {
		xlink = nval( getRequestParameter( lnk ),xlink );
	}

	/**
	 * 【TAG】ダイレクトジャンプを使用するかどうかを指定します（初期値：false)。
	 *
	 * @og.tag
	 * ダイレクトジャンプ(指定の画面へ直接飛ばす)は、簡易ワークフロー的に、画面への
	 * リンクを作成する場合に使用します。一般には、メール本文に記述して、指定の画面まで
	 * 直接飛ばす場合に使用します。
	 * このリンクは、その画面の index.jsp に直接ジャンプします。
	 *
	 * gamenId を指定しなければ、この画面が指定されたと考えます。
	 * target を指定しなければ、"_blank" を指定します。
	 * msg がなければ(または、BODY部が空の場合)、画面名にこのリンクを作成します。
	 * デフォルトは、 "false"（使用しない） です。
	 *
	 * @og.rev 3.7.1.0 (2005/04/26) 新規追加
	 *
	 * @param	directJump ダイレクトジャンプを使用するかどうか（使用する：true/使用しない：false)
	 */
	public void setUseDirectJump( final String directJump ) {
		useDirectJump = nval( getRequestParameter( directJump ),useDirectJump );
	}

	/**
	 * 【TAG】リクエストキャッシュ情報を使用するかどうかを指定します（初期値：true)。
	 *
	 * @og.tag
	 * リクエストキャッシュ情報は、戻るリンクで画面を差表示(command=RENEW)する場合に
	 * 最後に検索に利用された情報をキャッシュしておき、再利用する場合に利用する値です。
	 * ダイレクトジャンプ機能(useDirectJum=true)の場合のみ有効です。
	 * 戻るリンクによる再表示は、各画面自身が持っている情報を利用しますが、
	 * ダイレクトジャンプ等で、すでにキャッシュが存在していない（セッションが異なっている）
	 * 場合に、リンク情報に直接紐付けて、再検索させる必要がある場合に利用します。
	 * このキャッシュは、keys,vals による値の再設定で、書き換えることが可能です。
	 *
	 * デフォルトは、 "true"（使用する） です。
	 *
	 * @og.rev 3.7.1.0 (2005/04/26) 新規追加
	 *
	 * @param	useCache リクエストキャッシュ情報を使用するかどうか（使用する：true/使用しない：false)
	 */
	public void setUseRequestCache( final String useCache ) {
		useRequestCache = nval( getRequestParameter( useCache ),useRequestCache );
	}

	/**
	 * 【TAG】リクエストキャッシュ情報使用時に使用しない(削除する)キーをCSV形式で複数指定します。
	 *
	 * @og.tag
	 * リクエストキャッシュ情報には、一般に検索時に使用されたすべての引数が
	 * 使用されます。この引数を、転送先画面では、使用しないケースがあり、その場合に、
	 * 指定のキー情報を、削除します。
	 * 分解方法は、CSV変数を先に分解してから、getRequestParameter で値を取得します。
	 * こうしないとデータ自身にカンマを持っている場合に分解をミスる為です。
	 *
	 * @og.rev 3.7.1.0 (2005/04/26) 新規追加
	 *
	 * @param	key String 使用しない(削除する)キーをCSV形式で複数指定
	 */
	public void setDelKeys( final String key ) {
		delKeys = getCSVParameter( key );
	}

	/**
	 * 【TAG】ダイレクトジャンプリンク時のハッシュ化されたURLの有効期間を日数を指定します。
	 *
	 * @og.tag
	 * ダイレクトジャンプリンクの引数を作成する場合、有効期限を指定できます。
	 * ここでは、設定時刻からの有効期間を日数指定します。
	 * 実際には、GE17のvalidDateカラムに設定される、日付と現在日を比較して、
	 * 有効かどうかを判断します。
	 * ゼロが指定された場合は、制限なしに設定されます。マイナスの場合は、初期値が使われます。
	 * 初期値は、システムリソースの LINK_VALID_DURATION_DAYS で、設定します。
	 *
	 * @og.rev 4.0.0 (2005/09/30) 新規追加
	 *
	 * @param	validDura String URLの有効期間日数
	 */
	public void setValidDuration( final String validDura ) {
		validDuration = nval( getRequestParameter( validDura ),validDuration );
	}

	/**
	 * 【TAG】リンク(アンカータグ)ではなく、href 属性をそのまま返すかどうかを指定します（初期値：false)。
	 *
	 * @og.tag
	 * 通常のリンクは、アンカータグ(aタグ)の状態で出力されます。その場合、HTML上では
	 * リンクとして表示されます。このフラグをtrue にセットすると、リンク文字列
	 * そのものが出力されます。これは、リンク文字列を作成してDBに登録する場合などに
	 * 使用します。
	 *
	 * デフォルトは、 "false"（リンク） です。
	 *
	 * @og.rev 4.0.0 (2005/11/30) 新規追加
	 *
	 * @param	flag href 属性をそのまま返すかどうか（href 属性：true/リンク：false)
	 */
	public void setHrefOnly( final String flag ) {
		hrefOnly = nval( getRequestParameter( flag ),hrefOnly );
	}

	/**
	 * 【TAG】リンクアドレスにURLチェック用の確認キーを付加するかを指定します。（初期値：false)。
	 *
	 * @og.tag
	 * この属性は、URLCheckFilter(org.opengion.hayabusa.filter.URLCheckFilter)と合わせて使用します。
	 * trueが指定された場合、リンクURLのパラメーターに、URL確認用のパラメーターが出力されます。
	 * このパラメーターを、URLCheckFilterが復号化し、正式なアクセスであるかどうかの判定を行います。
	 *
	 * デフォルトは、 "false"（確認キーを出力しない） です。
	 *
	 * @og.rev 4.3.7.0 (2009/06/01) 新規追加
	 *
	 * @param	flag 暗号化するかかどうか
	 * @see org.opengion.hayabusa.filter.URLCheckFilter
	 */
	public void setUseURLCheck( final String flag ) {
		useURLCheck = nval( getRequestParameter( flag ),useURLCheck );
	}

	/**
	 * 【TAG】URLチェック用の確認キーに対して、リンクを開くことのできるユーザーIDを指定します。（初期値：ログインユーザー)
	 *
	 * @og.tag
	 * URLチェック用の確認キーに対して、リンクを開くことのできるユーザーIDを
	 * カンマ区切りで指定します。
	 * "*"が指定された場合、ユーザーチェックは行われません。
	 *
	 * デフォルトは、 ログインユーザーです。
	 *
	 * @og.rev 4.3.8.0 (2009/08/01) 新規追加
	 *
	 * @param	user リンクにアクセス可能なユーザー一覧
	 */
	public void setUrlCheckUser( final String user ) {
		urlCheckUser = nval( getRequestParameter( user ),urlCheckUser );
	}

	/**
	 * 【TAG】URLチェック用の確認キーに対して、リンクの有効期限を指定します。（初期値：60分）
	 *
	 * @og.tag
	 * URLチェック用の確認キーに対して、リンクが生成されてからの有効期限を分単位で設定します。
	 * 属性値の先頭に'D'をつけることで、日付指定を行うことができます。
	 * (D20090612 -> 20090612まで(当日を含む)は有効)
	 *
	 * デフォルトは、現在時間を基準として+ "1440"（1日） です。
	 *
	 * @og.rev 4.3.8.0 (2009/08/01) 新規追加
	 *
	 * @param	time リンクの有効期限(分)
	 */
	public void setUrlCheckTime( final String time ) {
		String tm = getRequestParameter( time );

		if( tm != null && tm.length() > 0 && tm.charAt( 0 ) == 'D' ) {
			if( tm.length() < 9 ) {
				String errMsg = "urlCheckTime属性の日付はYYYYMMDD形式で指定して下さい。";
				throw new HybsSystemException( errMsg );
			}
			Calendar cal = new GregorianCalendar(
					Integer.valueOf( tm.substring( 1, 5 ) )
					,Integer.valueOf( tm.substring( 5, 7 ) ) - 1
					,Integer.valueOf( tm.substring( 7, 9 ) ) );
			urlCheckTime = cal.getTimeInMillis() + (long)1000 * 60 * 60 * 24 - System.currentTimeMillis();
		}
		else {
			urlCheckTime = nval( tm, (long)60*24 ) * 1000 * 60;
		}
	}

	/**
	 * このオブジェクトの文字列表現を返します。
	 * 基本的にデバッグ目的に使用します。
	 *
	 * @return このクラスの文字列表現
	 */
	public String toString() {
		return org.opengion.fukurou.util.ToString.title( this.getClass().getName() )
				.println( "VERSION"			,VERSION		)
				.println( "column"			,column			)
				.println( "onLink"			,onLink			)
				.println( "fileCheck"		,fileCheck		)
				.println( "filePath"		,filePath		)
				.println( "baseURL"			,baseURL		)
				.println( "fileURL"			,fileURL		)
				.println( "gamenId"			,gamenId		)
				.println( "useGamenId"		,useGamenId		)
				.println( "markList"		,markList		)
				.println( "xlink"			,xlink			)
				.println( "useDirectJump"	,useDirectJump	)
				.println( "useRequestCache"	,useRequestCache)
				.println( "hrefOnly"		,hrefOnly		)
				.println( "validDuration"	,validDuration	)
				.println( "keys"			,keys			)
				.println( "vals"			,vals			)
				.println( "delKeys"			,delKeys		)
				.println( "useURLCheck"		,useURLCheck	)
				.println( "urlCheckUser"	,urlCheckUser	)
				.println( "urlCheckTime"	,urlCheckTime	)
				.println( "Other..."		,getAttributes().getAttribute() )
				.fixForm().toString() ;
	}
}
